import { fromString, toString } from "uint8arrays";
import { MIN_MESSAGE_ID, encodeId } from "./ids.js";
export class MerkleIndex {
    db;
    static defaultPageSize = 4096;
    constructor(db) {
        this.db = db;
    }
    async *entries(options = {}) {
        let lowerBound = { gte: MIN_MESSAGE_ID };
        while (true) {
            const results = await this.db.query("$messages", {
                orderBy: { id: "asc" },
                select: { id: true, hash: true },
                where: { id: lowerBound },
                limit: options.pageSize ?? MerkleIndex.defaultPageSize,
            });
            if (results.length === 0) {
                break;
            }
            for (const { id, hash } of results) {
                yield [encodeId(id), { hash: fromString(hash, "hex") }];
            }
            lowerBound = { gt: results[results.length - 1].id };
        }
    }
    static encodeNodeId = (level, key) => {
        const l = level.toString(16).padStart(2, "0");
        if (key === null) {
            return l;
        }
        else {
            const k = toString(key, "hex");
            return `${l}:${k}`;
        }
    };
    static encodeNode = (node) => {
        const id = MerkleIndex.encodeNodeId(node.level, node.key);
        return { id, hash: node.hash };
    };
    static decodeNode = ({ id, hash }) => {
        const [l, k] = id.split(":");
        const level = parseInt(l, 16);
        const key = k ? fromString(k, "hex") : null;
        return { level, key, hash };
    };
}
