import { toString } from "uint8arrays";
import { Mode } from "@canvas-js/okra";
import { Tree as MemoryTree } from "@canvas-js/okra-memory";
import { ModelDB } from "@canvas-js/modeldb-pg";
import { AbstractGossipLog } from "../AbstractGossipLog.js";
import { MerkleIndex } from "../MerkleIndex.js";
export class GossipLog extends AbstractGossipLog {
    db;
    tree;
    static async open(connectionConfig, { clear, ...init }) {
        const db = await ModelDB.initialize({
            connectionConfig,
            models: { ...init.schema, ...AbstractGossipLog.schema },
            clear: clear,
        });
        const messageCount = await db.count("$messages");
        const merkleIndex = new MerkleIndex(db);
        const start = performance.now();
        const tree = await MemoryTree.fromEntries({ mode: Mode.Index }, merkleIndex.entries());
        const root = await tree.read((txn) => txn.getRoot());
        const delta = performance.now() - start;
        const gossipLog = new GossipLog(db, tree, init);
        gossipLog.log(`build in-memory merkle tree (root %d:%s, %d entries, %dms)`, root.level, toString(root.hash, "hex"), messageCount, Math.round(delta));
        return gossipLog;
    }
    constructor(db, tree, init) {
        super(init);
        this.db = db;
        this.tree = tree;
    }
}
